/*****************************************************
Project :
Date    : 29.09.2009
Author  : Sergey

Chip type           : ATmega88
Program type        : Application
Clock frequency     : 8,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 256
*****************************************************/

#include <mega88.h>
	#ifndef __SLEEP_DEFINED__
	#define __SLEEP_DEFINED__
	.EQU __se_bit=0x01
	.EQU __sm_mask=0x0E
	.EQU __sm_adc_noise_red=0x02
	.EQU __sm_powerdown=0x04
	.EQU __sm_powersave=0x06
	.EQU __sm_standby=0x0C
	.SET power_ctrl_reg=smcr
	#endif

// Standard Input/Output functions
#include <stdio.h>
#include <delay.h>
// SPI functions
#include <spi.h>
#include "sdio.c"
#define CS PORTB.2
#define MOSI PORTB.3
#define SCK PORTB.5

/* Definitions for MMC/SDC command */
#define CMD0	(0x40+0)	/* GO_IDLE_STATE */
#define CMD1	(0x40+1)	/* SEND_OP_COND (MMC) */
#define	ACMD41	(0xC0+41)	/* SEND_OP_COND (SDC) */
#define CMD8	(0x40+8)	/* SEND_IF_COND */
#define CMD9	(0x40+9)	/* SEND_CSD */
#define CMD10	(0x40+10)	/* SEND_CID */
#define CMD12	(0x40+12)	/* STOP_TRANSMISSION */
#define ACMD13	(0xC0+13)	/* SD_STATUS (SDC) */
#define CMD16	(0x40+16)	/* SET_BLOCKLEN */
#define CMD17	(0x40+17)	/* READ_SINGLE_BLOCK */
#define CMD18	(0x40+18)	/* READ_MULTIPLE_BLOCK */
#define CMD23	(0x40+23)	/* SET_BLOCK_COUNT (MMC) */
#define	ACMD23	(0xC0+23)	/* SET_WR_BLK_ERASE_COUNT (SDC) */
#define CMD24	(0x40+24)	/* WRITE_BLOCK */
#define CMD25	(0x40+25)	/* WRITE_MULTIPLE_BLOCK */
#define CMD55	(0x40+55)	/* APP_CMD */
#define CMD58	(0x40+58)	/* READ_OCR */

#define SELECT()	PORTB.2 = 0	/* PB3: MMC CS = L */
#define	DESELECT()	PORTB.2 = 1	/* PB3: MMC CS = H */

char send_CMD(char cmd,unsigned long int arg);
void init_SD(void);

char buffer[512];//sector buffer

/////// Print ULONG value to UART ///////////
void print_long(unsigned long val)
{
        unsigned char i,arr[7];

        for(i=0;i<7;i++) arr[i]=0;

        while(val>=1000000) {val-=1000000;++arr[0];};
        while(val>=100000) {val-=100000;++arr[1];};
        while(val>=10000) {val-=10000;++arr[2];};
        while(val>=1000) {val-=1000;++arr[3];};
        while(val>=100) {val-=100;++arr[4];};
        while(val>=10) {val-=10;++arr[5];};
        while(val>=1) {val-=1;++arr[6];};

        for(i=0;i<7;i++) putchar(arr[i]+48);
        putchar('\n');
}

////////// Transmitt sector to UART ////////////////
void tx_sector(void)
{
        unsigned int i;

        for(i=0;i<512;i++) putchar(buffer[i]);
}

/// Load single sector into RAM buffer[512] ///
char load_sector(unsigned long adr )
{
        unsigned int i;
        unsigned char n;

        n=10;
        while((send_CMD(CMD17,adr)!=0) && n) --n;
        if(!n) return 1;

        while(spi(0xFF)!=0xFE);//wait for start of transmission

        for(i=0;i<512;i++)
                {
                buffer[i]=spi(0xFF);
                };
        spi(0xFF);//
        spi(0xFF);//read 2 bytes CRC
        return 0;
}

//////// Init SPI mode for SD card //////
void init_SD(void)
{
        unsigned char i;
start:
        SCK=1;
        CS=1;
        MOSI=1;

        for(i=0;i<100;i++)
                spi(0xFF);
        CS=0;

        send_CMD(CMD0,0);//Reset card
        /*if(send_CMD(CMD0,0)==0x01)
                putsf("0x01 = CMD0 OK");
        else
                putsf("CMD0 incorrect");

        putsf("Sending CMD1"); */
        i=0;
        while((send_CMD(CMD1,0)!=0x00)&&(i<250))++i;//Set SPI mode
        if(i==250) goto start;//If lailed then again
        //putsf("0x00 = CMD1 OK");
}

//static
char wait_ready (void)
{
	char res;

	spi(0xFF);
	do
		res = spi(0xFF);
	while (res != 0xFF);

	return res;
}

char send_CMD(char cmd,unsigned long int arg)
{
	char n, res;


	if (cmd & 0x80) {	/* ACMD<n> is the command sequense of CMD55-CMD<n> */
		cmd &= 0x7F;
		res = send_CMD(CMD55, 0);
		if (res > 1) return res;
	}

	/* Select the card and wait for ready */
	DESELECT();
	SELECT();
	if (wait_ready() != 0xFF) return 0xFF;

	/* Send command packet */
	spi(cmd);		        /* Start + Command index */
	spi((char)(arg >> 24));		/* Argument[31..24] */
	spi((char)(arg >> 16));		/* Argument[23..16] */
	spi((char)(arg >> 8));		/* Argument[15..8] */
	spi((char)arg);			/* Argument[7..0] */
	n = 0x01;			/* Dummy CRC + Stop */
	if (cmd == CMD0) n = 0x95;	/* Valid CRC for CMD0(0) */
	if (cmd == CMD8) n = 0x87;	/* Valid CRC for CMD8(0x1AA) */
	spi(n);

	/* Receive command response */
	n = 100;			/* Wait for a valid response in timeout of 10 attempts */
	do {
		res = spi(0xFF);
	} while ((res & 0x80) && --n);

	return res;			/* Return with the response value */
}
#include "fat.c"
unsigned long pbr_adr, fat_base,root_base, cluster_base;
unsigned long number_of_fat,sectors_per_fat,reserved;
unsigned long root_dir_entry,root_entry_size,sectors_per_cluster;


void get_FAT(void)
{
        //unsigned char response;
        //unsigned long tmp[4];
        /////////////////////////////////////////////
        load_sector(0);//load MBR sector

        //pbr_adr=((unsigned long) buffer[456])*65536;
        //pbr_adr+=((unsigned long)buffer[455])*256;
        //pbr_adr+=buffer[454];

        pbr_adr= ((unsigned long)buffer[0x01C8])<<16;
        pbr_adr+=((unsigned long)buffer[0x01C7])<<8;
        pbr_adr+=buffer[0x01C6];

        printf("PBR abress:");
        print_long(pbr_adr);
        /////////////////////////////////////////////
        load_sector(pbr_adr);//load PBR sector

        sectors_per_cluster=buffer[0x0D];
        printf("sector per cluster:");
        print_long(sectors_per_cluster);

        reserved=((unsigned long)buffer[0x0F])*256+buffer[0x0E];
        printf("reserved sectors:");
        print_long(reserved);

        number_of_fat=buffer[0x10];
        printf("number of fat:");
        print_long(number_of_fat);

        root_dir_entry=buffer[0x11]+(((unsigned long) buffer[0x12])*256);
        printf("root dir entry:");
        print_long(root_dir_entry);

        sectors_per_fat=buffer[0x16]+((unsigned long) buffer[0x17])*256;
        printf("sectors per fat:");
        print_long(sectors_per_fat);
        ////////////////////////////////////////////
        fat_base=pbr_adr+reserved;
        root_base=fat_base+number_of_fat*sectors_per_fat;
        cluster_base=root_base+root_dir_entry*root_entry_size-2*sectors_per_cluster;

}
// Declare your global variables here

void main(void)
{
// Declare your local variables here
unsigned long block_pointer=0,i,n;
// Crystal Oscillator division factor: 1
#pragma optsize-
CLKPR=0x80;
CLKPR=0x00;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

// Input/Output Ports initialization
// Port B initialization
// Func7=In Func6=In Func5=In Func4=Out Func3=In Func2=Out Func1=In Func0=In
// State7=T State6=T State5=T State4=0 State3=T State2=1 State1=T State0=T
PORTB=0x04;
DDRB=0b00101100;//0x14;

// Port C initialization
// Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In
// State6=T State5=T State4=T State3=T State2=T State1=T State0=T
PORTC=0x00;
DDRC=0x00;

// Port D initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=Out Func0=In
// State7=T State6=T State5=T State4=T State3=T State2=T State1=0 State0=T
PORTD=0x00;
DDRD=0x02;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: Timer 0 Stopped
// Mode: Normal top=FFh
// OC0A output: Disconnected
// OC0B output: Disconnected
TCCR0A=0x00;
TCCR0B=0x00;
TCNT0=0x00;
OCR0A=0x00;
OCR0B=0x00;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: Timer 1 Stopped
// Mode: Normal top=FFFFh
// OC1A output: Discon.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer 1 Overflow Interrupt: Off
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
TCCR1A=0x00;
TCCR1B=0x00;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// Timer/Counter 2 initialization
// Clock source: System Clock
// Clock value: Timer 2 Stopped
// Mode: Normal top=FFh
// OC2A output: Disconnected
// OC2B output: Disconnected
ASSR=0x00;
TCCR2A=0x00;
TCCR2B=0x00;
TCNT2=0x00;
OCR2A=0x00;
OCR2B=0x00;

// External Interrupt(s) initialization
// INT0: Off
// INT1: Off
// Interrupt on any change on pins PCINT0-7: Off
// Interrupt on any change on pins PCINT8-14: Off
// Interrupt on any change on pins PCINT16-23: Off
EICRA=0x00;
EIMSK=0x00;
PCICR=0x00;

// Timer/Counter 0 Interrupt(s) initialization
TIMSK0=0x00;
// Timer/Counter 1 Interrupt(s) initialization
TIMSK1=0x00;
// Timer/Counter 2 Interrupt(s) initialization
TIMSK2=0x00;

// USART initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// USART Receiver: Off
// USART Transmitter: On
// USART0 Mode: Asynchronous
// USART Baud Rate: 9600
UCSR0A=0x00;
UCSR0B=0x08;
UCSR0C=0x06;
UBRR0H=0x00;
UBRR0L=0x33;

// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;
ADCSRB=0x00;

// SPI initialization
// SPI Type: Slave
// SPI Clock Rate: 2000,000 kHz
// SPI Clock Phase: Cycle Half
// SPI Clock Polarity: Low
// SPI Data Order: MSB First
SPCR=0x53;//(62,5KHz)//40;
SPSR=0x00;
//125KHz
//SPCR=0x52;
//SPSR=0x00;

delay_ms(1000);

init_SD();

//get_FAT();
//send_CMD(CMD18,0);//Multiple block read
for(i=0;i<=1000;i++)
{
if(load_sector(i))
        {
        printf("\rread error, sector#");
        print_long(i);
        };
//delay_ms(20);
//tx_sector();
};

while (1)
      {
      // Place your code here

      };
}
