#define CS PORTB.2
#define MOSI PORTB.3
#define SCK PORTB.5

/* Definitions for MMC/SDC command */
#define CMD0	(0x40+0)	/* GO_IDLE_STATE */
#define CMD1	(0x40+1)	/* SEND_OP_COND (MMC) */
#define	ACMD41	(0xC0+41)	/* SEND_OP_COND (SDC) */
#define CMD8	(0x40+8)	/* SEND_IF_COND */
#define CMD9	(0x40+9)	/* SEND_CSD */
#define CMD10	(0x40+10)	/* SEND_CID */
#define CMD12	(0x40+12)	/* STOP_TRANSMISSION */
#define ACMD13	(0xC0+13)	/* SD_STATUS (SDC) */
#define CMD16	(0x40+16)	/* SET_BLOCKLEN */
#define CMD17	(0x40+17)	/* READ_SINGLE_BLOCK */
#define CMD18	(0x40+18)	/* READ_MULTIPLE_BLOCK */
#define CMD23	(0x40+23)	/* SET_BLOCK_COUNT (MMC) */
#define	ACMD23	(0xC0+23)	/* SET_WR_BLK_ERASE_COUNT (SDC) */
#define CMD24	(0x40+24)	/* WRITE_BLOCK */
#define CMD25	(0x40+25)	/* WRITE_MULTIPLE_BLOCK */
#define CMD55	(0x40+55)	/* APP_CMD */
#define CMD58	(0x40+58)	/* READ_OCR */

#define SELECT()	PORTB.2 = 0	/* PB3: MMC CS = L */
#define	DESELECT()	PORTB.2 = 1	/* PB3: MMC CS = H */

char send_CMD(char cmd,unsigned long int arg);
void init_SD(void); 

char buffer[512];//sector buffer

/////// Print ULONG value to UART ///////////
void print_long(unsigned long val)
{
        unsigned char i,arr[7];
        
        for(i=0;i<7;i++) arr[i]=0;
        
        while(val>=1000000) {val-=1000000;++arr[0];};
        while(val>=100000) {val-=100000;++arr[1];};
        while(val>=10000) {val-=10000;++arr[2];};
        while(val>=1000) {val-=1000;++arr[3];};
        while(val>=100) {val-=100;++arr[4];};
        while(val>=10) {val-=10;++arr[5];};
        while(val>=1) {val-=1;++arr[6];};
        
        for(i=0;i<7;i++) putchar(arr[i]+48);
        putchar('\n');
}

////////// Transmitt sector to UART ////////////////
void tx_sector(void)
{
        unsigned int i;
        
        for(i=0;i<512;i++) putchar(buffer[i]);
}
     
/// Load single sector into RAM buffer[512] ///
char load_sector(unsigned long adr )
{ 
        unsigned int i;
        unsigned char n;
        
        n=10;
        while((send_CMD(CMD17,adr)!=0) && n) --n;
        if(!n) return 1;
        
        while(spi(0xFF)!=0xFE);//wait for start of transmission
        
        for(i=0;i<512;i++)
                {        
                buffer[i]=spi(0xFF);
                };
        spi(0xFF);//
        spi(0xFF);//read 2 bytes CRC
        return 0;
}

//////// Init SPI mode for SD card //////
void init_SD(void)
{ 
        unsigned char i;
start:        
        SCK=1;
        CS=1;
        MOSI=1;
        
        for(i=0;i<100;i++)
                spi(0xFF);
        CS=0;
        
        send_CMD(CMD0,0);//Reset card
        /*if(send_CMD(CMD0,0)==0x01)
                putsf("0x01 = CMD0 OK");
        else
                putsf("CMD0 incorrect");
        
        putsf("Sending CMD1"); */
        i=0;
        while((send_CMD(CMD1,0)!=0x00)&&(i<250))++i;//Set SPI mode 
        if(i==250) goto start;//If lailed then again    
        //putsf("0x00 = CMD1 OK");
}

//static
char wait_ready (void)
{
	char res;

	spi(0xFF);
	do
		res = spi(0xFF);
	while (res != 0xFF);

	return res;
}

char send_CMD(char cmd,unsigned long int arg)
{
	char n, res;


	if (cmd & 0x80) {	/* ACMD<n> is the command sequense of CMD55-CMD<n> */
		cmd &= 0x7F;
		res = send_CMD(CMD55, 0);
		if (res > 1) return res;
	}

	/* Select the card and wait for ready */
	DESELECT();
	SELECT();
	if (wait_ready() != 0xFF) return 0xFF;

	/* Send command packet */
	spi(cmd);		        /* Start + Command index */
	spi((char)(arg >> 24));		/* Argument[31..24] */
	spi((char)(arg >> 16));		/* Argument[23..16] */
	spi((char)(arg >> 8));		/* Argument[15..8] */
	spi((char)arg);			/* Argument[7..0] */	
	n = 0x01;			/* Dummy CRC + Stop */
	if (cmd == CMD0) n = 0x95;	/* Valid CRC for CMD0(0) */
	if (cmd == CMD8) n = 0x87;	/* Valid CRC for CMD8(0x1AA) */
	spi(n);

	/* Receive command response */
	n = 100;			/* Wait for a valid response in timeout of 10 attempts */
	do {
		res = spi(0xFF);
	} while ((res & 0x80) && --n);

	return res;			/* Return with the response value */        
}